{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: Childwin.pas, released 11 Mar 2001.

The Initial Developer of the Original Code is: Jrgen Gntherodt
(jguentherodt@users.sourceforge.net).

Portions created by Jrgen Gntherodt are Copyright (C) 2001 Jrgen Gntherodt.
All Rights Reserved.

Alternatively, the contents of this file may be used under the terms of the
GNU General Public License Version 2 or later (the "GPL"), in which case
the provisions of the GPL are applicable instead of those above.
If you wish to allow use of your version of this file only under the terms
of the GPL and not to allow others to use your version of this file
under the MPL, indicate your decision by deleting the provisions above and
replace them with the notice and other provisions required by the GPL.
If you do not delete the provisions above, a recipient may use your version
of this file under either the MPL or the GPL.

$Id: CHILDWIN.PAS,v 1.3 2001/03/17 04:34:59 jguentherodt Exp $

You may retrieve the latest version of this file at the Open Perl IDE
home page, located at http://www.lost-sunglasses.de
-------------------------------------------------------------------------------}
unit Childwin;

interface

uses
  Windows, Classes, Graphics, Forms, Controls, StdCtrls, SynEdit, ExtCtrls,
  SynEditHighlighter, SynHighlighterPerl, sysUtils, iniFiles,
  SynEditTypes, Messages, breakpoints;

type
  TMDIChild = class(TForm)
    pnlChild: TPanel;
    SynEdit: TSynEdit;
    SynPerlSyn1: TSynPerlSyn;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SynEditSpecialLineColors(Sender: TObject; Line: Integer;
      var Special: Boolean; var FG, BG: TColor);
    procedure SynEditGutterClick(Sender: TObject; X, Y, Line: Integer;
      mark: TSynEditMark);
    procedure SynEditMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure SynEditClick(Sender: TObject);
    procedure SynEditChange(Sender: TObject);
    procedure SynEditStatusChange(Sender: TObject;
      Changes: TSynStatusChanges);
    procedure FormActivate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure SynEditMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    m_slBreakpoints: TStringList;
    m_nCurrentLine: Integer;
    m_nErrorLine: Integer;
    m_sFileName: String;
//    function GetBreakpoints: String;
    procedure SetBreakpoints(s: String);
//    function GetBreakpointCount: Integer;
    function IsPointInSelection(Value: TPoint): boolean;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWindowHandle(const Params: TCreateParams); override;
    procedure CreateWnd; override;
    procedure CreateHandle; override;
    procedure WndProc(var Message: TMessage); override;
  public
    { Public declarations }
    procedure SetCurrentLine(nLine: Integer);
    procedure SetErrorLine(nLine: Integer);
    function SaveFile: String;
    function SaveFileAs(sFileName: String): String;
    procedure OpenFile(sFileName: String);
//    procedure SendBreakpointsToDebugger;
//    procedure LoadBreakpointsFromIni;
//    procedure SaveBreakpointsToIni;
    procedure AddBreakpoint(Sender: TObject; sFileName: String; nLine: Integer; sCondition: String);
    procedure DeleteBreakpoint(Sender: TObject; sFileName: String; nLine: Integer; sCondition: String);
    constructor Create(AOwner: TComponent); override;
    procedure AfterConstruction; override;
    property FileName: String Read m_sFileName;
    property Breakpoints: String Write SetBreakpoints;
//    property BreakpointCount: Integer Read GetBreakpointCount;
  end;

implementation

uses Main;

{$R *.DFM}


constructor TMDIChild.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;


procedure TMDIChild.AfterConstruction;
begin
  inherited AfterConstruction;
  if MainForm.MDIChildCount = 1 then begin
    ShowWindow(Handle, SW_HIDE);
    ShowWindow(Handle, SW_MAXIMIZE);
  end;
end;


procedure TMDIChild.WndProc(var Message: TMessage);
begin
  inherited WndProc(Message); exit;
  if (Message.Msg = WM_SHOWWINDOW) and (Message.WParam <> 0)
//  if (False)
  then begin

    if MainForm.MDIChildCount = 1 then begin
      ShowWindow(Handle, SW_SHOWMAXIMIZED);
      Message.Result := 0
    end else begin
      inherited WndProc(Message);
    end;

  end else begin
    inherited WndProc(Message);
  end;
end;


procedure TMDIChild.CreateHandle;
begin
  inherited CreateHandle;
end;


procedure TMDIChild.CreateWindowHandle(const Params: TCreateParams);
begin
  inherited CreateWindowHandle(Params);
end;


procedure TMDIChild.CreateWnd;
begin
  inherited CreateWnd;
end;


procedure TMDIChild.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
//  Params.Style := Params.Style or WS_MAXIMIZE;
end;


procedure TMDIChild.AddBreakpoint(Sender: TObject; sFileName: String; nLine: Integer; sCondition: String);
begin
  m_slBreakpoints.Add(IntToStr(nLine));
  SynEdit.InvalidateLine(nLine);
  if Sender = Self
    then MainForm.DispatchBPChangeEvent(Sender, m_sFileName, nLine, '', False);
end;


procedure TMDIChild.DeleteBreakpoint(Sender: TObject; sFileName: String; nLine: Integer; sCondition: String);
var
  idx: Integer;
begin
  idx := m_slBreakpoints.IndexOf(IntToStr(nLine));
  if idx > -1 then m_slBreakpoints.Delete(idx);
  SynEdit.InvalidateLine(nLine);
  if Sender = Self
    then MainForm.DispatchBPChangeEvent(Sender, m_sFileName, nLine, '', True);
end;


procedure TMDIChild.SetBreakpoints(s: String);
var
  i: Integer;
  bpl: TBreakpointList;
begin
  bpl := TBreakpointList.Create('temp');
  bpl.AsString := s;
  for i := 0 to bpl.Count - 1 do
    m_slBreakpoints.Add(IntToStr(bpl[i].Line));
  bpl.Free;
end;


procedure TMDIChild.OpenFile(sFileName: String);
begin
  if sFileName = '' then begin
    m_sFileName := '<new file ' + IntToStr(MainForm.MDIChildCount) + '>';
    Caption := m_sFileName;
  end else begin
    m_sFileName := sFileName;
    SynEdit.Lines.LoadFromFile(sFileName);
    Caption := sFileName;
  end;
end;


function TMDIChild.SaveFileAs(sFileName: String): String;
begin
  if (Pos('<new file', sFileName) > 0) or (sFileName = '') then begin
    if Pos('<new file', m_sFileName) > 0
      then MainForm.SaveDialog.FileName := 'noname.pl'
      else MainForm.SaveDialog.FileName := m_sFileName;
    if MainForm.SaveDialog.Execute
      then sFileName := MainForm.SaveDialog.FileName
      else exit;
  end;
  if Pos('<new file', sFileName) = 0 then begin
    m_sFileName := sFileName;
    Caption := m_sFileName;
    SynEdit.Lines.SaveToFile(m_sFileName);
    SynEdit.ClearUndo;
    SynEditStatusChange(nil, []);
    if MainForm.tcFiles.TabIndex > -1 then begin
      MainForm.tcFiles.Tabs[MainForm.tcFiles.TabIndex] :=
        ExtractRelativePath(MainForm.m_sAppPath, m_sFileName);
    end;
    if UpperCase(ExtractFileExt(m_sFileName)) = '.PL'
      then MainForm.ChangeProject(m_sFileName);
  end;
  Result := sFileName;
end;



function TMDIChild.SaveFile: String;
begin
  Result := SaveFileAs(m_sFileName);
end;



procedure TMDIChild.SetErrorLine(nLine: Integer);
var
  nOldLine: Integer;
begin
  nOldLine := m_nErrorLine;
  m_nErrorLine := nLine;
  if nOldLine <> -1 then SynEdit.InvalidateLine(nOldLine);
  if m_nErrorLine > -1 then begin
    SynEdit.InvalidateLine(m_nErrorLine);
    SynEdit.CaretXY := Point(0, nLine);
    SynEdit.EnsureCursorPosVisible;
  end;
end;


procedure TMDIChild.SetCurrentLine(nLine: Integer);
var
  nOldLine: Integer;
begin
  nOldLine := m_nCurrentLine;
  m_nCurrentLine := nLine;
  if nOldLine <> -1 then SynEdit.InvalidateLine(nOldLine);
  if m_nCurrentLine > -1 then begin
    SynEdit.InvalidateLine(m_nCurrentLine);
    SynEdit.CaretXY := Point(0, nLine);
    SynEdit.EnsureCursorPosVisible;
  end;
end;

procedure TMDIChild.FormClose(Sender: TObject; var Action: TCloseAction);
var
  idx: Integer;
begin
  idx := MainForm.tcFiles.Tabs.IndexOf(ExtractRelativePath(MainForm.m_sAppPath, FileName));
  if idx > -1 then begin
    MainForm.tcFiles.Tabs.Delete(idx);
    if MainForm.tcFiles.Tabs.Count = 0 then MainForm.ChangeProject('');
  end;
  Action := caFree;
end;

procedure TMDIChild.FormCreate(Sender: TObject);
begin
  m_nCurrentLine := -1;
  m_nErrorLine := -1;
  m_slBreakpoints := TStringList.Create;
  m_slBreakpoints.Sorted := True;
  m_slBreakpoints.Duplicates := dupIgnore;
end;

procedure TMDIChild.FormDestroy(Sender: TObject);
begin
  m_slBreakpoints.Free;
end;

procedure TMDIChild.SynEditSpecialLineColors(Sender: TObject;
  Line: Integer; var Special: Boolean; var FG, BG: TColor);
begin
  if (Line = m_nCurrentLine) or (Line = m_nErrorLine) then begin
    if m_slBreakpoints.IndexOf(IntToStr(Line)) = -1 then begin
      if Line = m_nCurrentLine then begin
        FG := clWhite;
        BG := clNavy;
      end else begin
        FG := clWhite;
        BG := clMaroon;
      end;
    end else begin
      FG := clWhite;
      BG := clGreen;
    end;
    Special := True;
  end else begin
    if m_slBreakpoints.IndexOf(IntToStr(Line)) > -1 then begin
      FG := clWhite;
      BG := clRed;
      Special := True;
    end;
  end;
end;

procedure TMDIChild.SynEditGutterClick(Sender: TObject; X, Y,
  Line: Integer; mark: TSynEditMark);
var
  idx: Integer;
begin
  if Sender = nil then begin
    idx := m_slBreakpoints.IndexOf(IntToStr(Line));
    if (idx > -1) and (Line > 0)
      then DeleteBreakpoint(Self, m_sFileName, Line, '')
      else AddBreakpoint(Self, m_sFileName, Line, '');
  end;
end;




function TMDIChild.IsPointInSelection(Value: TPoint): boolean;
var
  ptBegin, ptEnd: TPoint;
begin
  with SynEdit do begin
    ptBegin := BlockBegin;
    ptEnd := BlockEnd;
    if (Value.Y >= ptBegin.Y) and (Value.Y <= ptEnd.Y) and
      ((ptBegin.Y <> ptEnd.Y) or (ptBegin.X <> ptEnd.X))
      then begin
      if SelectionMode = smLine then
        Result := TRUE
      else if (SelectionMode = smColumn) then begin
        if (ptBegin.X > ptEnd.X) then
          Result := (Value.X >= ptEnd.X) and (Value.X < ptBegin.X)
        else if (ptBegin.X < ptEnd.X) then
          Result := (Value.X >= ptBegin.X) and (Value.X < ptEnd.X)
        else
          Result := FALSE;
      end else
        Result := ((Value.Y > ptBegin.Y) or (Value.X >= ptBegin.X)) and
          ((Value.Y < ptEnd.Y) or (Value.X < ptEnd.X));
    end else
      Result := FALSE;
  end;
end;



procedure TMDIChild.SynEditMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  if MainForm.m_ExecutionState = ecsDebug then begin
    if SynEdit.SelAvail
    and IsPointInSelection(SynEdit.PixelsToRowColumn(Point(x, y))) then begin
      SynEdit.Hint := SynEdit.SelText;
    end else begin
      SynEdit.Hint := SynEdit.GetWordAtRowCol(SynEdit.PixelsToRowColumn(Point(x, y)))
    end
  end else SynEdit.Hint := '';
end;

procedure TMDIChild.SynEditClick(Sender: TObject);
begin
  SetErrorLine(-1);
end;

procedure TMDIChild.SynEditChange(Sender: TObject);
begin
  SetErrorLine(-1);
end;

procedure TMDIChild.SynEditStatusChange(Sender: TObject;
  Changes: TSynStatusChanges);
begin
  if SynEdit.CanUndo
    then Caption := m_sFileName + '*'
    else Caption := m_sFileName;
end;

procedure TMDIChild.FormActivate(Sender: TObject);
var
  idx: Integer;
begin
  idx := MainForm.tcFiles.Tabs.IndexOf(ExtractRelativePath(MainForm.m_sAppPath, FileName));
  if (idx > -1) and (idx <> MainForm.tcFiles.TabIndex) then begin
    MainForm.tcFiles.TabIndex := idx;
    MainForm.tcFilesChange(nil);
  end;
end;

procedure TMDIChild.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
  nRes: Integer;
begin
  if SynEdit.CanUndo then begin
    nRes := Application.MessageBox(PChar('Save changes to ' + m_sFileName + ' ?'),
      'Open Perl IDE', MB_YESNOCANCEL);
    if nRes = IDYES then SaveFile;
    CanClose := nRes <> IDCANCEL;
  end;
end;

procedure TMDIChild.SynEditMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  nLine: Integer;
begin
  if (x < SynEdit.Gutter.RealGutterWidth(SynEdit.CharWidth) - 2)
  and (not SynEdit.SelAvail) then begin
    nLine := SynEdit.PixelsToRowColumn(Point(X, Y)).Y;
    SynEditGutterClick(nil, x, y, nLine, nil);
  end;
end;

end.
