{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: PerlDebugger2.pas, released 11 Mar 2001.

The Initial Developer of the Original Code is: Jrgen Gntherodt
(jguentherodt@users.sourceforge.net).

Portions created by Jrgen Gntherodt are Copyright (C) 2001 Jrgen Gntherodt.
All Rights Reserved.

Alternatively, the contents of this file may be used under the terms of the
GNU General Public License Version 2 or later (the "GPL"), in which case
the provisions of the GPL are applicable instead of those above.
If you wish to allow use of your version of this file only under the terms
of the GPL and not to allow others to use your version of this file
under the MPL, indicate your decision by deleting the provisions above and
replace them with the notice and other provisions required by the GPL.
If you do not delete the provisions above, a recipient may use your version
of this file under either the MPL or the GPL.

$Id: PerlDebugger2.pas,v 1.3 2001/03/17 04:34:59 jguentherodt Exp $

You may retrieve the latest version of this file at the Open Perl IDE
home page, located at http://www.lost-sunglasses.de
-------------------------------------------------------------------------------}
unit PerlDebugger2;

interface

uses
  message_interface, winsock, clProcess, windows, sysUtils, Classes, syncObjs;

type
  TPerlDebugger = class(TInterfacedObject, IMessageDispatcher)
  private
    m_MessageReceiver: IMessageReceiver;
    m_sAppPath: String;
    m_hListenSocket: TSocket;
    m_hConnectionSocket: TSocket;
    m_nPort: Integer;
    m_clp: TCLProcess;
    procedure InternalStart(sProgramName, sParams: String; bBreakOnFirstLine: Boolean);
    procedure InternalTerminate;
    procedure CreateDebugScript(bSingleStep: Boolean);
    function ReceiveReport(dwReportToWaitFor: DWord): DWord;
    procedure SendCommand(sCommand: String);
    function InternalListen: DWord;
    procedure HandleCLEvent(Sender: TObject; sData: String; bError, bTerminate: Boolean);
    procedure HandleResponse(nMsgID: Integer; nErrorCode: Integer);
    procedure HandleReport(nMsgID: Integer; sData: String);
  protected
    function SendMessage(nMsgID: Integer; sData: String): Integer;
    procedure SetOnReceiveMessage(rcv: IMessageReceiver);
    function GetOnReceiveMessage: IMessageReceiver;
    procedure SendToStdIn(sData: String);
  public
    constructor Create(MessageReceiver: IMessageReceiver);
    destructor Destroy; override;
  end;

implementation


function ListenThreadFunc(p: Pointer): DWord; stdcall;
begin
  Result := TPerlDebugger(p).InternalListen;
end;


////////////////////////////////////////////////////////////////////////////////
//  TPerlDebugger = class(TInterfacedObject, IMessageDispatcher)
////////////////////////////////////////////////////////////////////////////////
constructor TPerlDebugger.Create(MessageReceiver: IMessageReceiver);
var
  sa: SOCKADDR_IN;
  i, nNameLen: Integer;
begin
  inherited Create;
  m_MessageReceiver := MessageReceiver;
  m_sAppPath := ExtractFilePath(ParamStr(0));
  // Create listening socket
  m_hListenSocket := socket(PF_INET, SOCK_STREAM, IPPROTO_IP);
  with sa do begin
    sin_family := AF_INET;
    sin_port := 0; // dynamic port reservation
    sin_addr.S_un_b.s_b1 := #127;
    sin_addr.S_un_b.s_b2 := #0;
    sin_addr.S_un_b.s_b3 := #0;
    sin_addr.S_un_b.s_b4 := #1;
    for i := 0 to 7 do sin_zero[i] := #0;
  end;
  bind(m_hListenSocket, sa, SizeOf(sa));
  FillChar(sa, 0, SizeOf(sa));
  // read port address
  nNameLen := SizeOf(sa);
  getsockname(m_hListenSocket, sa, nNameLen);
  m_nPort := ntohs(sa.sin_port);
end;


destructor TPerlDebugger.Destroy;
begin
  if m_clp <> nil then SendMessage(DM_STOP_REQ, '');
  // Close listening socket
  if m_hListenSocket <> INVALID_SOCKET then begin
    closesocket(m_hListenSocket);
    m_hListenSocket := INVALID_SOCKET;
  end;
  m_hListenSocket := INVALID_SOCKET;
  inherited Destroy;
end;


procedure TPerlDebugger.SendToStdIn(sData: String);
begin
  if m_clp <> nil then m_clp.Input(sData);
end;


procedure TPerlDebugger.InternalTerminate;
begin
  WaitForSingleObject(m_clp.ProcessHandle, INFINITE);
  m_clp.WaitFor;
  m_clp.Free;
  m_clp := nil;
  closesocket(m_hConnectionSocket);
  m_hConnectionSocket := INVALID_SOCKET;
end;


procedure TPerlDebugger.InternalStart(sProgramName, sParams: String; bBreakOnFirstLine: Boolean);
var
  sCommandLine: String;
  hListenThread: THandle;
  dwThreadID: DWord;
  sInclude1, sInclude2: String;
begin
  hListenThread := CreateThread(nil, 0, @ListenThreadFunc, Self, 0, dwThreadID);
  if m_clp <> nil then exit;
  CreateDebugScript(bBreakOnFirstLine);

  sInclude1 := '.';

  sInclude2 := ExtractFilePath(sProgramName);
  if copy(sInclude2, Length(sInclude2), 1) = '\'
    then sInclude2 := copy(sInclude2, 1, Length(sInclude2) - 1);

  sCommandLine := Format('perl -d -I"%s" -I"%s" "%s" %s',
    [sInclude1, sInclude2, sProgramName, sParams]);

  if m_clp <> nil then m_clp.Free;
  m_clp := TCLProcess.Create(sCommandLine, m_sAppPath, HandleCLEvent);

  WaitForSingleObject(hListenThread, INFINITE);
end;


procedure TPerlDebugger.CreateDebugScript(bSingleStep: Boolean);
var
  sl: TStringList;
  i: Integer;
  nSingleStep: Integer;
begin
  if bSingleStep then nSingleStep := 1 else nSingleStep := 0;
  sl := TStringList.Create;
  sl.LoadFromFile(m_sAppPath + 'dbTemplate.txt');
  for i := 0 to sl.Count - 1 do begin
    sl[i] := StringReplace(sl[i], '*PORT*', IntToStr(m_nPort), []);
    sl[i] := StringReplace(sl[i], '*SINGLE_STEP*', IntToStr(nSingleStep), []);
  end;
  sl.SaveToFile(m_sAppPath + 'perl5db.pl');
  sl.Free;
end;


function TPerlDebugger.ReceiveReport(dwReportToWaitFor: DWord): DWord;
var
  lDataLength: Longint;
  i, code: Integer;
  dw: DWord;
  sData: String;
  sl: TStringList;
  nBytesRead: Integer;
begin
  sl := TStringList.Create;
  Result := 0;
  while (Result <> DM_ERROR_REP) and (Result <> dwReportToWaitFor) do begin
    // Response from perl script is always in format <length of data><data>
    // Data can be interpreted as StringList with first entry = MsgID
    i := 0;
    nBytesRead := 1;
    lDataLength := 0;
    while (i < 4) and (nBytesRead > 0) do begin
      nBytesRead := recv(m_hConnectionSocket, Pointer(Longint(@lDataLength) + i)^, 4 - i, 0);
      i := i + nBytesRead;
    end;
    if lDataLength > 0 then begin
      SetLength(sData, lDataLength);
      i := 0;
      while i < lDataLength do begin
        i := i + recv(m_hConnectionSocket, sData[i + 1], lDataLength - i, 0);
      end;
    end else sData := '';
    Result := DM_ERROR_REP;
    sl.Text := sData;
    if sl.Count > 0 then begin
      val(sl[0], dw, code);
      if code = 0 then begin
        Result := dw;
        sl.Delete(0);
      end;
    end;
    sData := sl.Text;
    if copy(sData, Length(sData) - 1, 2) = #13#10
      then sData := copy(sData, 1, Length(sData) - 2);
    HandleReport(Result, sData);
  end;
  sl.Free;
end;


procedure TPerlDebugger.SendCommand(sCommand: String);
var
  nDataLength: Integer;
begin
  // Data to perl script is always sent in format <data length><data>
  nDataLength := Length(sCommand);
  send(m_hConnectionSocket, nDataLength, 4, 0);
  if nDataLength > 0 then send(m_hConnectionSocket, sCommand[1], nDataLength, 0);
end;


function TPerlDebugger.InternalListen: DWord;
begin
  listen(m_hListenSocket, 0);
  m_hConnectionSocket := accept(m_hListenSocket, nil, nil);
  Result := 0;
end;


procedure TPerlDebugger.HandleCLEvent(Sender: TObject; sData: String; bError, bTerminate: Boolean);
begin
  if bTerminate then begin
    m_MessageReceiver.HandleTerminate;
  end else begin
    if bError
      then m_MessageReceiver.ReceiveFromStdErr(sData)
      else m_MessageReceiver.ReceiveFromStdOut(sData);
  end;
end;


procedure TPerlDebugger.HandleResponse(nMsgID: Integer; nErrorCode: Integer);
begin
  if m_MessageReceiver <> nil
    then m_MessageReceiver.HandleMessage(DM_CMD_RSP, IntToStr(nMsgID) + #13#10 + IntToStr(nErrorCode));
end;


procedure TPerlDebugger.HandleReport(nMsgID: Integer; sData: String);
begin
  if m_MessageReceiver <> nil
    then m_MessageReceiver.HandleMessage(nMsgID, sData);
end;


function TPerlDebugger.SendMessage(nMsgID: Integer; sData: String): Integer;
var
  sl: TStringList;
  dwReportMsg: DWord;
  sModuleName, sLine, sCondition: String;
  bCompileOnly: Boolean;
  sRequestID: String;
  sCode: String;
begin
  Result := 0;
  sl := TStringList.Create;
  case nMsgID of
    DM_START_REQ: begin
      sl.Text := sData;
      if sl.Count = 4 then begin
        // DM_START_REQ
        // -ProgramName
        // -Params
        // -SingleStepMode
        // -Compile only
        HandleResponse(nMsgID, Result);
        bCompileOnly := (sl[3] <> '') and (sl[3] <> '0') and (UpperCase(sl[3]) <> 'FALSE');
        InternalStart(sl[0], sl[1],
          (sl[2] <> '') and (sl[2] <> '0') and (UpperCase(sl[2]) <> 'FALSE'));
        dwReportMsg := ReceiveReport(DM_START_REP);
        // Receive first line report or abort session
        if (dwReportMsg = DM_START_REP) and (not bCompileOnly)
          then ReceiveReport(DM_LINE_REP)
          else SendMessage(DM_STOP_REQ, '');
      end else begin
        Result := DEC_INVALID_PARAMS;
        HandleResponse(nMsgID, Result);
      end;
    end;
    DM_STOP_REQ: begin
      if m_clp = nil then begin
        Result := DEC_NO_SESSION;
        HandleResponse(nMsgID, Result);
      end else begin
        HandleResponse(nMsgID, Result);
        SendCommand('STOP');
        ReceiveReport(DM_STOP_REP);
        InternalTerminate;
      end;
    end;
    DM_RUN_REQ: begin
      HandleResponse(nMsgID, Result);
      SendCommand('RUN');
      ReceiveReport(DM_LINE_REP);
    end;
    DM_STEP_OVER_REQ: begin
      HandleResponse(nMsgID, Result);
      SendCommand('STEP_OVER');
      ReceiveReport(DM_LINE_REP);
    end;
    DM_STEP_INTO_REQ: begin
      HandleResponse(nMsgID, Result);
      SendCommand('STEP_INTO');
      ReceiveReport(DM_LINE_REP);
    end;
    DM_CHANGE_BP_REQ: begin
      sl.Text := sData;
      if sl.Count = 4 then begin
        HandleResponse(nMsgID, Result);
        sModuleName := sl[0];
        sLine := sl[1];
        sCondition := sl[2];
        if (sl[3] <> '') and (sl[3] <> '0')
        and (UpperCase(sl[3]) <> 'FALSE') then begin
          SendCommand('ADD_BREAKPOINT='
            + '<' + sModulename + #9 + sLine + #9+ sCondition);
        end else begin
          SendCommand('DELETE_BREAKPOINT='
            + '<' + sModulename + #9 + sLine + #9 + sCondition);
        end;
      end else begin
        Result := DEC_INVALID_PARAMS;
        HandleResponse(nMsgID, Result);
      end;
    end;
    DM_CLEAR_BP_REQ: begin
      HandleResponse(nMsgID, Result);
      SendCommand('CLEAR_BREAKPOINTS=<' + sData);
    end;
    DM_EVAL_CODE_REQ: begin
      sl.Text := sData;
      if sl.Count = 2 then begin
        HandleResponse(nMsgID, Result);
        sRequestID := sl[0];
        sCode := sl[1];
        SendCommand('EVAL_CODE=' + sRequestID + '<' + sCode);
        ReceiveReport(DM_EVAL_CODE_REP);
      end else begin
        Result := DEC_INVALID_PARAMS;
        HandleResponse(nMsgID, Result);
      end;
    end;
    DM_CALLSTACK_REQ: begin
      HandleResponse(nMsgID, Result);
      SendCommand('CALLSTACK=' + sData);
      ReceiveReport(DM_CALLSTACK_REP);
    end;
  else
    Result := DEC_UNKNOWN_MESSAGE;
  end;

  sl.Free;
end;


procedure TPerlDebugger.SetOnReceiveMessage(rcv: IMessageReceiver);
begin
   m_MessageReceiver := rcv;
end;


function TPerlDebugger.GetOnReceiveMessage: IMessageReceiver;
begin
  Result := m_MessageReceiver;
end;


var
  wsData: WSAData;
  
initialization
  WSAStartup(258, wsData);
finalization
  WSACleanup;
end.
