{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: message_interface.pas, released 11 Mar 2001.

The Initial Developer of the Original Code is: Jrgen Gntherodt
(jguentherodt@users.sourceforge.net).

Portions created by Jrgen Gntherodt are Copyright (C) 2001 Jrgen Gntherodt.
All Rights Reserved.

Alternatively, the contents of this file may be used under the terms of the
GNU General Public License Version 2 or later (the "GPL"), in which case
the provisions of the GPL are applicable instead of those above.
If you wish to allow use of your version of this file only under the terms
of the GPL and not to allow others to use your version of this file
under the MPL, indicate your decision by deleting the provisions above and
replace them with the notice and other provisions required by the GPL.
If you do not delete the provisions above, a recipient may use your version
of this file under either the MPL or the GPL.

$Id: message_interface.pas,v 1.3 2001/03/17 04:34:59 jguentherodt Exp $

You may retrieve the latest version of this file at the Open Perl IDE
home page, located at http://www.lost-sunglasses.de
-------------------------------------------------------------------------------}
unit message_interface;

interface

type
  IMessageReceiver = interface ['{3D36F000-FF89-11D4-8FD8-8618A6D8704A}']
    function HandleMessage(nMsgID: Integer; sData: String): Integer;
    function ReceiveFromStdOut(sData: String): Integer;
    function ReceiveFromStdErr(sData: String): Integer;
    function HandleTerminate: Integer;
  end;


  IMessageDispatcher = interface ['{3D36F001-FF89-11D4-8FD8-8618A6D8704A}']
    function SendMessage(nMsgID: Integer; sData: String): Integer;
    procedure SendToStdIn(sData: String);
    procedure SetOnReceiveMessage(rcv: IMessageReceiver);
    function GetOnReceiveMessage: IMessageReceiver;

    property OnReceiveMessage: IMessageReceiver Read GetOnReceiveMessage Write SetOnReceiveMessage;
  end;

  // Debug messages
  const
    // Response messages
    DM_CMD_RSP           = $00000000;

    // Request messages
    DM_START_REQ         = $00000100;
    DM_STOP_REQ          = $00000200;
    DM_RUN_REQ           = $00000300;
    DM_STEP_OVER_REQ     = $00000301;
    DM_STEP_INTO_REQ     = $00000302;
    DM_CHANGE_BP_REQ     = $00000400;
    DM_CLEAR_BP_REQ      = $00000401;
    DM_EVAL_CODE_REQ     = $00000500;
    DM_CALLSTACK_REQ     = $00000600;

    // Report messages
    DM_ERROR_REP         = $FFFFFFFF;
    DM_START_REP         = $00010100;
    DM_MODULE_LOADED_REP = $00010101;
    DM_STOP_REP          = $00010200;
    DM_LINE_REP          = $00010300;
    DM_EVAL_CODE_REP     = $00010500;
    DM_CALLSTACK_REP     = $00010600;

  // Error codes
    DEC_OK              = 0;
    DEC_UNKNOWN_MESSAGE = -1;
    DEC_INVALID_PARAMS  = -2;
    DEC_NO_SESSION      = -3;

function MsgToStr(dwMsgID: Cardinal): String;
function ErrorToStr(nErrorID: Integer): String;

implementation

function ErrorToStr(nErrorID: Integer): String;
begin
  case nErrorID of
    DEC_OK: Result := 'DEC_OK';
    DEC_UNKNOWN_MESSAGE: Result := 'DEC_UNKNOWN_MESSAGE';
    DEC_INVALID_PARAMS: Result := 'DEC_INVALID_PARAMS';
    DEC_NO_SESSION: Result := 'DEC_NO_SESSION';
  else
    Result := 'Unknown error';
  end;
end;

function MsgToStr(dwMsgID: Cardinal): String;
begin
  case dwMsgID of
    DM_START_REQ: Result := 'DM_START_REQ';
    DM_STOP_REQ: Result := 'DM_STOP_REQ';
    DM_RUN_REQ: Result := 'DM_RUN_REQ';
    DM_STEP_OVER_REQ: Result := 'DM_STEP_OVER_REQ';
    DM_STEP_INTO_REQ: Result := 'DM_STEP_INTO_REQ';
    DM_CHANGE_BP_REQ: Result := 'DM_CHANGE_BP_REQ';
    DM_CLEAR_BP_REQ: Result := 'DM_CLEAR_BP_REQ';
    DM_EVAL_CODE_REQ: Result := 'DM_EVAL_CODE_REQ';
    DM_CALLSTACK_REQ: Result := 'DM_CALLSTACK_REQ';
    DM_ERROR_REP: Result := 'DM_ERROR_REP';
    DM_START_REP: Result := 'DM_START_REP';
    DM_MODULE_LOADED_REP: Result := 'DM_MODULE_LOADED_REP';
    DM_STOP_REP: Result := 'DM_STOP_REP';
    DM_LINE_REP: Result := 'DM_LINE_REP';
    DM_EVAL_CODE_REP: Result := 'DM_EVAL_CODE_REP';
    DM_CALLSTACK_REP: Result := 'DM_CALLSTACK_REP';
  else
    Result := 'Unknown message';
  end;
end;

end.
