{-------------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: f_Variables.pas, released 11 Mar 2001.

The Initial Developer of the Original Code is: Jrgen Gntherodt
(jguentherodt@users.sourceforge.net).

Portions created by Jrgen Gntherodt are Copyright (C) 2001 Jrgen Gntherodt.
All Rights Reserved.

Alternatively, the contents of this file may be used under the terms of the
GNU General Public License Version 2 or later (the "GPL"), in which case
the provisions of the GPL are applicable instead of those above.
If you wish to allow use of your version of this file only under the terms
of the GPL and not to allow others to use your version of this file
under the MPL, indicate your decision by deleting the provisions above and
replace them with the notice and other provisions required by the GPL.
If you do not delete the provisions above, a recipient may use your version
of this file under either the MPL or the GPL.

$Id: f_Variables.pas,v 1.3 2001/03/17 04:34:59 jguentherodt Exp $

You may retrieve the latest version of this file at the Open Perl IDE
home page, located at http://www.lost-sunglasses.de
-------------------------------------------------------------------------------}
unit f_Variables;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Grids, SynEdit, ElTree, ElHeader;

  
type
  TfrmVariables = class(TForm)
    pnlClient: TPanel;
    Splitter1: TSplitter;
    pnlLocalVariables: TPanel;
    Panel3: TPanel;
    memoLocals: TMemo;
    pnlWatches: TPanel;
    Panel1: TPanel;
    eltWatches: TElTree;
    procedure eltWatchesDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure eltWatchesDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure eltWatchesKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure eltWatchesResize(Sender: TObject);
    procedure eltWatchesHeaderColumnResize(Sender: TObject;
      SectionIndex: Integer);
    procedure eltWatchesDblClick(Sender: TObject);
    procedure eltWatchesValidateInplaceEdit(Sender: TObject;
      Item: TElTreeItem; Section: TElHeaderSection; var Text: String;
      var Accept: Boolean);
    procedure eltWatchesItemExpand(Sender: TObject; Item: TElTreeItem);
    procedure eltWatchesTryEdit(Sender: TObject; Item: TElTreeItem;
      Section: TElHeaderSection; var CellType: TFieldTypes;
      var CanEdit: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure eltWatchesItemDeletion(Sender: TObject; Item: TElTreeItem);
    procedure eltWatchesItemLoad(Sender: TObject; Stream: TStream;
      Item: TElTreeItem);
  private
    { Private declarations }
    m_slItemLookup: TStringList;
    m_eltNew: TElTreeItem;
    procedure ParseTypeValue(Item: TElTreeItem; sTypeValue: String);
    function GetAsString: String;
    procedure SetAsString(s: String);
    procedure RefreshItems(Item: TElTreeItem);
  public
    { Public declarations }
    procedure UpdateVariable(sRequestID, sName, sValue: String);
    procedure EvaluateVariable(Item: TElTreeItem; sText: String);
    function HasMembers(Item: TElTreeItem): Boolean;
    function ExtractVariableFromCode(sCode: String): String;
    function GetNextHexDecode(var sEncodedString: String): String;
    property AsString: String Read GetAsString Write SetAsString;
  end;

var
  frmVariables: TfrmVariables;

implementation

uses Main;

{$R *.DFM}


procedure TfrmVariables.RefreshItems(Item: TElTreeItem);
var
  i: Integer;
begin
  Item.SubItems.Clear;
  Item.Collapse(True);
  Item.Clear;
  EvaluateVariable(Item, Item.GetFullNameEx('->', False));
  if Item.Expanded then begin
    for i := 0 to Item.Count - 1 do RefreshItems(Item.Item[i]);
  end;
end;


function TfrmVariables.GetAsString: String;
var
  ms: TMemoryStream;
begin
  ms := TMemoryStream.Create;
  try
    try eltWatches.Items.SaveToStream(ms); except end;
    SetLength(Result, 2 * ms.Size);
    BinToHex(PChar(ms.Memory), PChar(Result), ms.Size);
  finally
    ms.Free;
  end;
end;


procedure TfrmVariables.SetAsString(s: String);
var
  ms: TMemoryStream;
begin
  eltWatches.Items.Clear;
  m_eltNew := eltWatches.Items.AddItem(nil);
  if s = '' then exit;
  ms := TMemoryStream.Create;
  try
    ms.Size := Length(s) div 2;
    HexToBin(PChar(s), PChar(ms.Memory), ms.Size);
    ms.Seek(0, soFromBeginning);
    try eltWatches.Items.LoadFromStream(ms); except end;
  finally
    if eltWatches.Items.Count > 0
      then m_eltNew := eltWatches.Items[eltWatches.Items.Count - 1]
      else m_eltNew := eltWatches.Items.AddItem(nil);
    m_eltNew.Text := '';
    m_eltNew.SubItems.Clear;
    m_eltNew.Clear;
    ms.Free;
  end;
end;


function TfrmVariables.GetNextHexDecode(var sEncodedString: String): String;
var
  sLength: String;
  nLength: Integer;
begin
  sLength := copy(sEncodedString, 1, 8);
  nLength := StrToIntDef('$' + sLength, -1);
  if nLength > -1 then begin
    Result := copy(sEncodedString, Length(sLength) + 1, nLength);
    sEncodedString := copy(sEncodedString, Length(sLength) + nLength + 1, MAXINT);
  end else begin
    Result := '';
    sEncodedString := '';
  end;
end;


function TfrmVariables.ExtractVariableFromCode(sCode: String): String;
var
  iPos: Integer;
begin
  iPos := Pos('(\', sCode);
  if iPos > 0 then begin
    sCode := copy(sCode, iPos + 2, Length(sCode));
    if copy(sCode, Length(sCode), 1) = ')' then begin
      sCode := copy(sCode, 1, Length(sCode) - 1);
    end;
  end;
  Result := sCode;
end;


function TfrmVariables.HasMembers(Item: TElTreeItem): Boolean;
begin
  Result := (Pos('ARRAY', UpperCase(Item.SubItems[0])) > 0)
         or (Pos('HASH', UpperCase(Item.SubItems[0])) > 0)
         or (Pos('CLASS', UpperCase(Item.SubItems[0])) > 0);
end;


procedure TfrmVariables.ParseTypeValue(Item: TElTreeItem; sTypeValue: String);
var
  iPos: Integer;
  sType, sValue: String;
begin
  iPos := Pos(' = ', sTypeValue);
  if iPos > 0 then begin
    sType := copy(sTypeValue, 1, iPos - 1);
    sValue := copy(sTypeValue, iPos + 3, Length(sTypeValue));
    if Item <> nil then begin
      Item.SubItems.Add(sType);
      Item.SubItems.Add(sValue);
    end;
  end else begin
    Item.SubItems.Add(sTypeValue);
    Item.SubItems.Add(sTypeValue);
  end;
end;


procedure TfrmVariables.EvaluateVariable(Item: TElTreeItem; sText: String);
var
  sResponse: String;
begin
  sResponse := MainForm.BlockingEval(
      IntToStr(Longint(Item)) + #13#10 + 'ShortEval(\' + sText + ')');
  ParseTypeValue(Item, sResponse);
  if HasMembers(Item) and (Item.Count = 0) then eltWatches.Items.AddChild(Item, '<internal item>');
end;


procedure TfrmVariables.UpdateVariable(sRequestID, sName, sValue: String);
var
  item: TElTreeItem;
  idx: Integer;
begin
  idx := m_slItemLookup.IndexOf(sRequestID);
  if idx > -1 then begin
    item := TElTreeItem(m_slItemLookup.Objects[idx]);
    item.ColumnText.Clear;
    ParseTypeValue(item, sValue);
  end;
end;


procedure TfrmVariables.eltWatchesDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := Source is TSynEdit;
end;

procedure TfrmVariables.eltWatchesDragDrop(Sender, Source: TObject; X,
  Y: Integer);
var
  eltItem: TElTreeItem;
begin
  if Source is TSynEdit then begin
    m_eltNew.Delete;
    try
      eltItem := eltWatches.Items.AddItem(nil);
      m_slItemLookup.AddObject(IntToStr(Longint(eltItem)), eltItem);
      eltItem.Text := TSynEdit(Source).SelText;
      EvaluateVariable(eltItem, eltItem.Text);
    finally
      m_eltNew := eltWatches.Items.AddItem(nil);
    end;
  end;
end;

procedure TfrmVariables.eltWatchesKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if eltWatches.ItemFocused = nil then exit;
  case Key of
    13: RefreshItems(eltWatches.ItemFocused);
    46: if (eltWatches.ItemFocused.Level = 0)
        and (eltWatches.ItemFocused <> m_eltNew)
          then eltWatches.ItemFocused.Delete;
  else
  end;
end;

procedure TfrmVariables.eltWatchesResize(Sender: TObject);
begin
  if eltWatches.IsEditing then eltWatches.EndEdit(True);
  eltWatches.HeaderSections[2].Width := eltWatches.ClientWidth
    - eltWatches.HeaderSections[0].Width
    - eltWatches.HeaderSections[1].Width;
end;

procedure TfrmVariables.eltWatchesHeaderColumnResize(Sender: TObject;
  SectionIndex: Integer);
begin
  if eltWatches.IsEditing then eltWatches.EndEdit(True);
  eltWatches.HeaderSections[2].Width := eltWatches.ClientWidth
    - eltWatches.HeaderSections[0].Width
    - eltWatches.HeaderSections[1].Width;
end;

procedure TfrmVariables.eltWatchesDblClick(Sender: TObject);
var
  pt: TPoint;
  ItemPart: TSTItemPart;
  nHitColumn: Integer;
  it: TElTreeItem;
begin
  GetCursorPos(pt);
  pt := eltWatches.ScreenToClient(pt);
  it := eltWatches.GetItemAt(pt.x, pt.y, ItemPart, nHitColumn);
  if (it <> nil) and (nHitColumn = 0) then it.EditText;
end;

procedure TfrmVariables.eltWatchesValidateInplaceEdit(Sender: TObject;
  Item: TElTreeItem; Section: TElHeaderSection; var Text: String;
  var Accept: Boolean);
var
  sResponse: String;
begin
  Accept := Accept and (Text <> '');
  if (not Accept) then exit;
  if Section.Index = 0 then begin
    // Evaluate watch
    Item.SubItems.Clear;
    Item.Collapse(True);
    Item.Clear;
    EvaluateVariable(Item, Text);
    Item.Text := Text;
    if Item = m_eltNew then m_eltNew := eltWatches.Items.AddItem(nil);
  end else begin
    // Set watch to new value
    sResponse := MainForm.BlockingEval(
        IntToStr(Longint(Item)) + #13#10
      + Item.GetFullNameEx('->', False) + '=' + Text);
    Item.SubItems[1] := sResponse;
  end;
end;

procedure TfrmVariables.eltWatchesItemExpand(Sender: TObject;
  Item: TElTreeItem);
var
  sResponse: String;
  sl: TStringList;
  ti: TElTreeItem;
  sPath: String;
  sName, sTypeValue: String;
begin
  if (Item.ChildrenCount = 1) and (Item.Children[0].Text = '<internal item>') then begin
    sl := TStringList.Create;
    try
      sPath := Item.GetFullNameEx('->', False);
      Item.Clear;
      sResponse := MainForm.BlockingEval(
          IntToStr(Longint(Item)) + #13#10 + 'LongEval(\' + sPath + ')');

      sName := GetNextHexDecode(sResponse);
      while (sName <> '') do begin
        ti := eltWatches.Items.AddChild(Item, sName);
        m_slItemLookup.AddObject(IntToStr(Longint(ti)), ti);
        sTypeValue := GetNextHexDecode(sResponse);
        ParseTypeValue(ti, sTypeValue);
        if HasMembers(ti) then eltWatches.Items.AddChild(ti, '<internal item>');
        sName := GetNextHexDecode(sResponse);
      end;

    finally
      sl.Free;
    end;
  end;
end;


procedure TfrmVariables.eltWatchesTryEdit(Sender: TObject;
  Item: TElTreeItem; Section: TElHeaderSection; var CellType: TFieldTypes;
  var CanEdit: Boolean);
begin
  if Section.Index = 0 then CanEdit := Item.Parent = nil;
  if Section.Index = 2 then CanEdit := Item.SubItems[0] = 'scalar';
end;


procedure TfrmVariables.FormCreate(Sender: TObject);
begin
  m_eltNew := eltWatches.Items.AddItem(nil);
  m_slItemLookup := TStringList.Create;
  m_slItemLookup.Sorted := True;
end;


procedure TfrmVariables.FormDestroy(Sender: TObject);
begin
  m_slItemLookup.Free;
end;


procedure TfrmVariables.eltWatchesItemDeletion(Sender: TObject;
  Item: TElTreeItem);
var
  idx: Integer;
begin
  if csDestroying in ComponentState then exit;
  idx := m_slItemLookup.IndexOf(IntToStr(Longint(Item)));
  if idx > -1 then m_slItemLookup.Delete(idx);
end;

procedure TfrmVariables.eltWatchesItemLoad(Sender: TObject;
  Stream: TStream; Item: TElTreeItem);
begin
  m_slItemLookup.AddObject(IntToStr(Longint(Item)), Item);
end;

end.
